@@ -15,33 +15,25 @@
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
 
-"""B+Tree indices"""
+"""B+Tree indices."""
 
 from io import BytesIO
 
 from ..lazy_import import lazy_import
-lazy_import(globals(), """
+
+lazy_import(
+    globals(),
+    """
 import bisect
 import math
 import tempfile
 import zlib
-""")
-
-from .. import (
-    chunk_writer,
-    debug,
-    fifo_cache,
-    lru_cache,
-    osutils,
-    trace,
-    transport,
-    )
-from . import (
-    index,
-    static_tuple,
-    )
-from .index import _OPTION_NODE_REFS, _OPTION_KEY_ELEMENTS, _OPTION_LEN
+""",
+)
 
+from .. import chunk_writer, debug, fifo_cache, lru_cache, osutils, trace, transport
+from . import index as _mod_index
+from .index import _OPTION_KEY_ELEMENTS, _OPTION_LEN, _OPTION_NODE_REFS
 
 _BTSIGNATURE = b"B+Tree Graph Index 2\n"
 _OPTION_ROW_LENGTHS = b"row_lengths="
@@ -56,7 +48,7 @@ _PAGE_SIZE = 4096
 _NODE_CACHE_SIZE = 1000
 
 
-class _BuilderRow(object):
+class _BuilderRow:
     """The stored state accumulated while writing out a row in the index.
 
     :ivar spool: A temporary file used to accumulate nodes for this row
@@ -78,7 +70,7 @@ class _BuilderRow(object):
             self.spool.write(b"\x00" * _RESERVED_HEADER_BYTES)
         elif self.nodes == 1:
             # We got bigger than 1 node, switch to a temp file
-            spool = tempfile.TemporaryFile(prefix='bzr-index-row-')
+            spool = tempfile.TemporaryFile(prefix="bzr-index-row-")
             spool.write(self.spool.getvalue())
             self.spool = spool
         skipped_bytes = 0
@@ -88,8 +80,9 @@ class _BuilderRow(object):
         self.spool.writelines(byte_lines)
         remainder = (self.spool.tell() + skipped_bytes) % _PAGE_SIZE
         if remainder != 0:
-            raise AssertionError("incorrect node length: %d, %d"
-                                 % (self.spool.tell(), remainder))
+            raise AssertionError(
+                f"incorrect node length: {self.spool.tell()}, {remainder}"
+            )
         self.nodes += 1
         self.writer = None
 
@@ -107,7 +100,7 @@ class _LeafBuilderRow(_BuilderRow):
     """The stored state accumulated while writing out a leaf rows."""
 
 
-class BTreeBuilder(index.GraphIndexBuilder):
+class BTreeBuilder(_mod_index.GraphIndexBuilder):
     """A Builder for B+Tree based Graph indices.
 
     The resulting graph has the structure:
@@ -141,8 +134,9 @@ class BTreeBuilder(index.GraphIndexBuilder):
         :param spill_at: Optional parameter controlling the maximum number
             of nodes that BTreeBuilder will hold in memory.
         """
-        index.GraphIndexBuilder.__init__(self, reference_lists=reference_lists,
-                                         key_elements=key_elements)
+        _mod_index.GraphIndexBuilder.__init__(
+            self, reference_lists=reference_lists, key_elements=key_elements
+        )
         self._spill_at = spill_at
         self._backing_indices = []
         # A map of {key: (node_refs, value)}
@@ -152,7 +146,7 @@ class BTreeBuilder(index.GraphIndexBuilder):
         self._optimize_for_size = False
 
     def add_node(self, key, value, references=()):
-        """Add a node to the index.
+        r"""Add a node to the index.
 
         If adding the node causes the builder to reach its spill_at threshold,
         disk spilling will be triggered.
@@ -165,13 +159,13 @@ class BTreeBuilder(index.GraphIndexBuilder):
         :param value: The value to associate with the key. It may be any
             bytes as long as it does not contain \\0 or \\n.
         """
-        # Ensure that 'key' is a StaticTuple
-        key = static_tuple.StaticTuple.from_sequence(key).intern()
+        # Ensure that 'key' is a tuple.
+        key = tuple(key)
         # we don't care about absent_references
         node_refs, _ = self._check_key_ref_value(key, references, value)
         if key in self._nodes:
-            raise index.BadIndexDuplicateKey(key, self)
-        self._nodes[key] = static_tuple.StaticTuple(node_refs, value)
+            raise _mod_index.BadIndexDuplicateKey(key, self)
+        self._nodes[key] = (node_refs, value)
         if self._nodes_by_key is not None and self._key_length > 1:
             self._update_nodes_by_key(key, value, node_refs)
         if len(self._nodes) < self._spill_at:
@@ -192,22 +186,22 @@ class BTreeBuilder(index.GraphIndexBuilder):
         size 4x. On the fifth create a single new one, etc.
         """
         if self._combine_backing_indices:
-            (new_backing_file, size,
-             backing_pos) = self._spill_mem_keys_and_combine()
+            (new_backing_file, size, backing_pos) = self._spill_mem_keys_and_combine()
         else:
             new_backing_file, size = self._spill_mem_keys_without_combining()
         # Note: The transport here isn't strictly needed, because we will use
         #       direct access to the new_backing._file object
-        new_backing = BTreeGraphIndex(transport.get_transport_from_path('.'),
-                                      '<temp>', size)
+        new_backing = BTreeGraphIndex(
+            transport.get_transport_from_path("."), "<temp>", size
+        )
         # GC will clean up the file
         new_backing._file = new_backing_file
         if self._combine_backing_indices:
             if len(self._backing_indices) == backing_pos:
                 self._backing_indices.append(None)
             self._backing_indices[backing_pos] = new_backing
-            for backing_pos in range(backing_pos):
-                self._backing_indices[backing_pos] = None
+            for bp in range(backing_pos):
+                self._backing_indices[bp] = None
         else:
             self._backing_indices.append(new_backing)
         self._nodes = {}
@@ -225,9 +219,9 @@ class BTreeBuilder(index.GraphIndexBuilder):
                 break
             iterators_to_combine.append(backing.iter_all_entries())
         backing_pos = pos + 1
-        new_backing_file, size = \
-            self._write_nodes(self._iter_smallest(iterators_to_combine),
-                              allow_optimize=False)
+        new_backing_file, size = self._write_nodes(
+            self._iter_smallest(iterators_to_combine), allow_optimize=False
+        )
         return new_backing_file, size, backing_pos
 
     def add_nodes(self, nodes):
@@ -236,10 +230,10 @@ class BTreeBuilder(index.GraphIndexBuilder):
         :param nodes: An iterable of (key, node_refs, value) entries to add.
         """
         if self.reference_lists:
-            for (key, value, node_refs) in nodes:
+            for key, value, node_refs in nodes:
                 self.add_node(key, value, node_refs)
         else:
-            for (key, value) in nodes:
+            for key, value in nodes:
                 self.add_node(key, value)
 
     def _iter_mem_nodes(self):
@@ -256,8 +250,7 @@ class BTreeBuilder(index.GraphIndexBuilder):
 
     def _iter_smallest(self, iterators_to_combine):
         if len(iterators_to_combine) == 1:
-            for value in iterators_to_combine[0]:
-                yield value
+            yield from iterators_to_combine[0]
             return
         current_values = []
         for iterator in iterators_to_combine:
@@ -268,15 +261,18 @@ class BTreeBuilder(index.GraphIndexBuilder):
         last = None
         while True:
             # Decorate candidates with the value to allow 2.4's min to be used.
-            candidates = [(item[1][1], item) for item
-                          in enumerate(current_values) if item[1] is not None]
+            candidates = [
+                (item[1][1], item)
+                for item in enumerate(current_values)
+                if item[1] is not None
+            ]
             if not len(candidates):
                 return
             selected = min(candidates)
             # undecorate back to (pos, node)
             selected = selected[1]
             if last == selected[1][1]:
-                raise index.BadIndexDuplicateKey(last, self)
+                raise _mod_index.BadIndexDuplicateKey(last, self)
             last = selected[1][1]
             # Yield, with self as the index
             yield (self,) + selected[1][1:]
@@ -311,23 +307,26 @@ class BTreeBuilder(index.GraphIndexBuilder):
                     else:
                         optimize_for_size = False
                     internal_row.writer = chunk_writer.ChunkWriter(
-                        length, 0, optimize_for_size=optimize_for_size)
+                        length, 0, optimize_for_size=optimize_for_size
+                    )
                     internal_row.writer.write(_INTERNAL_FLAG)
-                    internal_row.writer.write(_INTERNAL_OFFSET
-                                              + b"%d\n" % rows[pos + 1].nodes)
+                    internal_row.writer.write(
+                        _INTERNAL_OFFSET + b"%d\n" % rows[pos + 1].nodes
+                    )
             # add a new leaf
             length = _PAGE_SIZE
             if rows[-1].nodes == 0:
                 length -= _RESERVED_HEADER_BYTES  # padded
             rows[-1].writer = chunk_writer.ChunkWriter(
-                length, optimize_for_size=self._optimize_for_size)
+                length, optimize_for_size=self._optimize_for_size
+            )
             rows[-1].writer.write(_LEAF_FLAG)
         if rows[-1].writer.write(line):
             # if we failed to write, despite having an empty page to write to,
             # then line is too big. raising the error avoids infinite recursion
             # searching for a suitably large page that will not be found.
             if new_leaf:
-                raise index.BadIndexKey(string_key)
+                raise _mod_index.BadIndexKey(string_key)
             # this key did not fit in the node:
             rows[-1].finish_node()
             key_line = string_key + b"\n"
@@ -346,8 +345,8 @@ class BTreeBuilder(index.GraphIndexBuilder):
             # division point, then we need a new root:
             if new_row:
                 # We need a new row
-                if 'index' in debug.debug_flags:
-                    trace.mutter('Inserting new global row.')
+                if "index" in debug.debug_flags:
+                    trace.mutter("Inserting new global row.")
                 new_row = _InternalBuilderRow()
                 reserved_bytes = 0
                 rows.insert(0, new_row)
@@ -355,13 +354,12 @@ class BTreeBuilder(index.GraphIndexBuilder):
                 new_row.writer = chunk_writer.ChunkWriter(
                     _PAGE_SIZE - _RESERVED_HEADER_BYTES,
                     reserved_bytes,
-                    optimize_for_size=self._optimize_for_size)
+                    optimize_for_size=self._optimize_for_size,
+                )
                 new_row.writer.write(_INTERNAL_FLAG)
-                new_row.writer.write(_INTERNAL_OFFSET
-                                     + b"%d\n" % (rows[1].nodes - 1))
+                new_row.writer.write(_INTERNAL_OFFSET + b"%d\n" % (rows[1].nodes - 1))
                 new_row.writer.write(key_line)
-            self._add_key(string_key, line, rows,
-                          allow_optimize=allow_optimize)
+            self._add_key(string_key, line, rows, allow_optimize=allow_optimize)
 
     def _write_nodes(self, node_iterator, allow_optimize=True):
         """Write node_iterator out as a B+Tree.
@@ -394,29 +392,33 @@ class BTreeBuilder(index.GraphIndexBuilder):
                 rows.append(_LeafBuilderRow())
             key_count += 1
             string_key, line = _btree_serializer._flatten_node(
-                node, self.reference_lists)
-            self._add_key(string_key, line, rows,
-                          allow_optimize=allow_optimize)
+                node, self.reference_lists
+            )
+            self._add_key(string_key, line, rows, allow_optimize=allow_optimize)
         for row in reversed(rows):
-            pad = (not isinstance(row, _LeafBuilderRow))
+            pad = not isinstance(row, _LeafBuilderRow)
             row.finish_node(pad=pad)
         lines = [_BTSIGNATURE]
-        lines.append(b'%s%d\n' % (_OPTION_NODE_REFS, self.reference_lists))
-        lines.append(b'%s%d\n' % (_OPTION_KEY_ELEMENTS, self._key_length))
-        lines.append(b'%s%d\n' % (_OPTION_LEN, key_count))
+        lines.append(b"%s%d\n" % (_OPTION_NODE_REFS, self.reference_lists))
+        lines.append(b"%s%d\n" % (_OPTION_KEY_ELEMENTS, self._key_length))
+        lines.append(b"%s%d\n" % (_OPTION_LEN, key_count))
         row_lengths = [row.nodes for row in rows]
-        lines.append(_OPTION_ROW_LENGTHS + ','.join(
-            map(str, row_lengths)).encode('ascii') + b'\n')
+        lines.append(
+            _OPTION_ROW_LENGTHS
+            + ",".join(map(str, row_lengths)).encode("ascii")
+            + b"\n"
+        )
         if row_lengths and row_lengths[-1] > 1:
-            result = tempfile.NamedTemporaryFile(prefix='bzr-index-')
+            result = tempfile.NamedTemporaryFile(prefix="bzr-index-")
         else:
             result = BytesIO()
         result.writelines(lines)
         position = sum(map(len, lines))
         if position > _RESERVED_HEADER_BYTES:
-            raise AssertionError("Could not fit the header in the"
-                                 " reserved space: %d > %d"
-                                 % (position, _RESERVED_HEADER_BYTES))
+            raise AssertionError(
+                "Could not fit the header in the"
+                f" reserved space: {position} > {_RESERVED_HEADER_BYTES}"
+            )
         # write the rows out:
         for row in rows:
             reserved = _RESERVED_HEADER_BYTES  # reserved space for first node
@@ -432,10 +434,10 @@ class BTreeBuilder(index.GraphIndexBuilder):
             copied_len = osutils.pumpfile(row.spool, result)
             if copied_len != (row.nodes - 1) * _PAGE_SIZE:
                 if not isinstance(row, _LeafBuilderRow):
-                    raise AssertionError("Incorrect amount of data copied"
-                                         " expected: %d, got: %d"
-                                         % ((row.nodes - 1) * _PAGE_SIZE,
-                                            copied_len))
+                    raise AssertionError(
+                        "Incorrect amount of data copied"
+                        f" expected: {(row.nodes - 1) * _PAGE_SIZE}, got: {copied_len}"
+                    )
         result.flush()
         size = result.tell()
         result.seek(0)
@@ -450,15 +452,14 @@ class BTreeBuilder(index.GraphIndexBuilder):
         return self._write_nodes(self.iter_all_entries())[0]
 
     def iter_all_entries(self):
-        """Iterate over all keys within the index
+        """Iterate over all keys within the index.
 
         :return: An iterable of (index, key, value, reference_lists). There is
             no defined order for the result iteration - it will be in the most
             efficient order for the index (in this case dictionary hash order).
         """
-        if 'evil' in debug.debug_flags:
-            trace.mutter_callsite(
-                3, "iter_all_entries scales with size of history.")
+        if "evil" in debug.debug_flags:
+            trace.mutter_callsite(3, "iter_all_entries scales with size of history.")
         # Doing serial rather than ordered would be faster; but this shouldn't
         # be getting called routinely anyway.
         iterators = [self._iter_mem_nodes()]
@@ -537,7 +538,7 @@ class BTreeBuilder(index.GraphIndexBuilder):
                 yield (self,) + node[1:]
         if self._key_length == 1:
             for key in keys:
-                index._sanity_check_key(self, key)
+                _mod_index._sanity_check_key(self, key)
                 try:
                     node = self._nodes[key]
                 except KeyError:
@@ -548,8 +549,7 @@ class BTreeBuilder(index.GraphIndexBuilder):
                     yield self, key, node[1]
             return
         nodes_by_key = self._get_nodes_by_key()
-        for entry in index._iter_entries_prefix(self, nodes_by_key, keys):
-            yield entry
+        yield from _mod_index._iter_entries_prefix(self, nodes_by_key, keys)
 
     def _get_nodes_by_key(self):
         if self._nodes_by_key is None:
@@ -561,7 +561,7 @@ class BTreeBuilder(index.GraphIndexBuilder):
                         key_dict = key_dict.setdefault(subkey, {})
                     key_dict[key[-1]] = key, value, references
             else:
-                for key, (references, value) in self._nodes.items():
+                for key, (_references, value) in self._nodes.items():
                     key_dict = nodes_by_key
                     for subkey in key[:-1]:
                         key_dict = key_dict.setdefault(subkey, {})
@@ -577,7 +577,8 @@ class BTreeBuilder(index.GraphIndexBuilder):
         return len(self._nodes) + sum(
             backing.key_count()
             for backing in self._backing_indices
-            if backing is not None)
+            if backing is not None
+        )
 
     def validate(self):
         """In memory index's have no known corruption at the moment."""
@@ -594,23 +595,24 @@ class BTreeBuilder(index.GraphIndexBuilder):
 class _LeafNode(dict):
     """A leaf node for a serialised B+Tree index."""
 
-    __slots__ = ('min_key', 'max_key', '_keys')
+    __slots__ = ("_keys", "max_key", "min_key")
 
     def __init__(self, bytes, key_length, ref_list_length):
         """Parse bytes to create a leaf node object."""
         # splitlines mangles the \r delimiters.. don't use it.
         key_list = _btree_serializer._parse_leaf_lines(
-            bytes, key_length, ref_list_length)
+            bytes, key_length, ref_list_length
+        )
         if key_list:
             self.min_key = key_list[0][0]
             self.max_key = key_list[-1][0]
         else:
             self.min_key = self.max_key = None
-        super(_LeafNode, self).__init__(key_list)
+        super().__init__(key_list)
         self._keys = dict(self)
 
     def all_items(self):
-        """Return a sorted list of (key, (value, refs)) items"""
+        """Return a sorted list of (key, (value, refs)) items."""
         items = sorted(self.items())
         return items
 
@@ -620,38 +622,34 @@ class _LeafNode(dict):
         return keys
 
 
-class _InternalNode(object):
+class _InternalNode:
     """An internal node for a serialised B+Tree index."""
 
-    __slots__ = ('keys', 'offset')
+    __slots__ = ("keys", "offset")
 
     def __init__(self, bytes):
         """Parse bytes to create an internal node object."""
         # splitlines mangles the \r delimiters.. don't use it.
-        self.keys = self._parse_lines(bytes.split(b'\n'))
+        self.keys = self._parse_lines(bytes.split(b"\n"))
 
     def _parse_lines(self, lines):
         nodes = []
         self.offset = int(lines[1][7:])
-        as_st = static_tuple.StaticTuple.from_sequence
         for line in lines[2:]:
-            if line == b'':
+            if line == b"":
                 break
-            # GZ 2017-05-24: Used to intern() each chunk of line as well, need
-            # to recheck performance and perhaps adapt StaticTuple to adjust.
-            nodes.append(as_st(line.split(b'\0')).intern())
+            nodes.append(tuple(line.split(b"\0")))
         return nodes
 
 
-class BTreeGraphIndex(object):
+class BTreeGraphIndex:
     """Access to nodes via the standard GraphIndex interface for B+Tree's.
 
     Individual nodes are held in a LRU cache. This holds the root node in
     memory except when very large walks are done.
     """
 
-    def __init__(self, transport, name, size, unlimited_cache=False,
-                 offset=0):
+    def __init__(self, transport, name, size, unlimited_cache=False, offset=0):
         """Create a B+Tree index object on the index name.
 
         :param transport: The transport to read data for the index from.
@@ -699,11 +697,12 @@ class BTreeGraphIndex(object):
             isinstance(self, type(other))
             and self._transport == other._transport
             and self._name == other._name
-            and self._size == other._size)
+            and self._size == other._size
+        )
 
     def __lt__(self, other):
         if isinstance(other, type(self)):
-            return ((self._name, self._size) < (other._name, other._size))
+            return (self._name, self._size) < (other._name, other._size)
         # Always sort existing indexes before ones that are still being built.
         if isinstance(other, BTreeBuilder):
             return True
@@ -757,8 +756,10 @@ class BTreeGraphIndex(object):
         Otherwise it will be computed based on the length of the index.
         """
         if self._size is None:
-            raise AssertionError('_compute_total_pages_in_index should not be'
-                                 ' called when self._size is None')
+            raise AssertionError(
+                "_compute_total_pages_in_index should not be"
+                " called when self._size is None"
+            )
         if self._root_node is not None:
             # This is the number of pages as defined by the header
             return self._row_offsets[-1]
@@ -780,19 +781,22 @@ class BTreeGraphIndex(object):
         :param offsets: The offsets to be read
         :return: A list of offsets to download
         """
-        if 'index' in debug.debug_flags:
-            trace.mutter('expanding: %s\toffsets: %s', self._name, offsets)
+        if "index" in debug.debug_flags:
+            trace.mutter("expanding: %s\toffsets: %s", self._name, offsets)
 
         if len(offsets) >= self._recommended_pages:
             # Don't add more, we are already requesting more than enough
-            if 'index' in debug.debug_flags:
-                trace.mutter('  not expanding large request (%s >= %s)',
-                             len(offsets), self._recommended_pages)
+            if "index" in debug.debug_flags:
+                trace.mutter(
+                    "  not expanding large request (%s >= %s)",
+                    len(offsets),
+                    self._recommended_pages,
+                )
             return offsets
         if self._size is None:
             # Don't try anything, because we don't know where the file ends
-            if 'index' in debug.debug_flags:
-                trace.mutter('  not expanding without knowing index size')
+            if "index" in debug.debug_flags:
+                trace.mutter("  not expanding without knowing index size")
             return offsets
         total_pages = self._compute_total_pages_in_index()
         cached_offsets = self._get_offsets_to_cached_pages()
@@ -801,12 +805,11 @@ class BTreeGraphIndex(object):
         if total_pages - len(cached_offsets) <= self._recommended_pages:
             # Read whatever is left
             if cached_offsets:
-                expanded = [x for x in range(total_pages)
-                            if x not in cached_offsets]
+                expanded = [x for x in range(total_pages) if x not in cached_offsets]
             else:
                 expanded = list(range(total_pages))
-            if 'index' in debug.debug_flags:
-                trace.mutter('  reading all unread pages: %s', expanded)
+            if "index" in debug.debug_flags:
+                trace.mutter("  reading all unread pages: %s", expanded)
             return expanded
 
         if self._root_node is None:
@@ -825,15 +828,16 @@ class BTreeGraphIndex(object):
                 # then it isn't worth expanding our request. Once we've read at
                 # least 2 nodes, then we are probably doing a search, and we
                 # start expanding our requests.
-                if 'index' in debug.debug_flags:
-                    trace.mutter('  not expanding on first reads')
+                if "index" in debug.debug_flags:
+                    trace.mutter("  not expanding on first reads")
                 return offsets
-            final_offsets = self._expand_to_neighbors(offsets, cached_offsets,
-                                                      total_pages)
+            final_offsets = self._expand_to_neighbors(
+                offsets, cached_offsets, total_pages
+            )
 
         final_offsets = sorted(final_offsets)
-        if 'index' in debug.debug_flags:
-            trace.mutter('expanded:  %s', final_offsets)
+        if "index" in debug.debug_flags:
+            trace.mutter("expanded:  %s", final_offsets)
         return final_offsets
 
     def _expand_to_neighbors(self, offsets, cached_offsets, total_pages):
@@ -859,16 +863,20 @@ class BTreeGraphIndex(object):
                 if first is None:
                     first, end = self._find_layer_first_and_end(pos)
                 previous = pos - 1
-                if (previous > 0 and
-                    previous not in cached_offsets and
-                    previous not in final_offsets and
-                        previous >= first):
+                if (
+                    previous > 0
+                    and previous not in cached_offsets
+                    and previous not in final_offsets
+                    and previous >= first
+                ):
                     next_tips.add(previous)
                 after = pos + 1
-                if (after < total_pages and
-                    after not in cached_offsets and
-                    after not in final_offsets and
-                        after < end):
+                if (
+                    after < total_pages
+                    and after not in cached_offsets
+                    and after not in final_offsets
+                    and after < end
+                ):
                     next_tips.add(after)
                 # This would keep us from going bigger than
                 # recommended_pages by only expanding the first offsets.
@@ -897,8 +905,9 @@ class BTreeGraphIndex(object):
         if self._root_node is None:
             self._get_root_node()
         if ref_list_num + 1 > self.node_ref_lists:
-            raise ValueError('No ref list %d, index has %d ref lists'
-                             % (ref_list_num, self.node_ref_lists))
+            raise ValueError(
+                f"No ref list {ref_list_num}, index has {self.node_ref_lists} ref lists"
+            )
         keys = set()
         refs = set()
         for node in self.iter_all_entries():
@@ -987,9 +996,8 @@ class BTreeGraphIndex(object):
             There is no defined order for the result iteration - it will be in
             the most efficient order for the index.
         """
-        if 'evil' in debug.debug_flags:
-            trace.mutter_callsite(
-                3, "iter_all_entries scales with size of history.")
+        if "evil" in debug.debug_flags:
+            trace.mutter_callsite(3, "iter_all_entries scales with size of history.")
         if not self.key_count():
             return
         if self._row_offsets[-1] == 1:
@@ -998,7 +1006,7 @@ class BTreeGraphIndex(object):
                 for key, (value, refs) in self._root_node.all_items():
                     yield (self, key, value, refs)
             else:
-                for key, (value, refs) in self._root_node.all_items():
+                for key, (value, _refs) in self._root_node.all_items():
                     yield (self, key, value)
             return
         start_of_leaves = self._row_offsets[-2]
@@ -1018,7 +1026,7 @@ class BTreeGraphIndex(object):
                     yield (self, key, value, refs)
         else:
             for _, node in nodes:
-                for key, (value, refs) in node.all_items():
+                for key, (value, _refs) in node.all_items():
                     yield (self, key, value)
 
     @staticmethod
@@ -1084,16 +1092,16 @@ class BTreeGraphIndex(object):
                         cur_keys.append(cur_in_key)
                         try:
                             cur_in_key = next(in_keys_iter)
-                        except StopIteration:
-                            raise InputDone
+                        except StopIteration as exc:
+                            raise InputDone from exc
                     # At this point cur_in_key must be >= cur_fixed_key
                 # step the cur_fixed_key until we pass the cur key, or walk off
                 # the end
                 while cur_in_key >= cur_fixed_key:
                     try:
                         cur_fixed_offset, cur_fixed_key = next(fixed_keys_iter)
-                    except StopIteration:
-                        raise FixedDone
+                    except StopIteration as exc:
+                        raise FixedDone from exc
         except InputDone:
             # We consumed all of the input, nothing more to do
             pass
@@ -1119,7 +1127,7 @@ class BTreeGraphIndex(object):
         # large queries are being made.
         keys_at_index = [(0, sorted(keys))]
 
-        for row_pos, next_row_start in enumerate(self._row_offsets[1:-1]):
+        for _row_pos, next_row_start in enumerate(self._row_offsets[1:-1]):
             node_indexes = [idx for idx, s_keys in keys_at_index]
             nodes = self._get_internal_nodes(node_indexes)
 
@@ -1128,8 +1136,9 @@ class BTreeGraphIndex(object):
                 node = nodes[node_index]
                 positions = self._multi_bisect_right(sub_keys, node.keys)
                 node_offset = next_row_start + node.offset
-                next_nodes_and_keys.extend([(node_offset + pos, s_keys)
-                                            for pos, s_keys in positions])
+                next_nodes_and_keys.extend(
+                    [(node_offset + pos, s_keys) for pos, s_keys in positions]
+                )
             keys_at_index = next_nodes_and_keys
         # We should now be at the _LeafNodes
         node_indexes = [idx for idx, s_keys in keys_at_index]
@@ -1225,8 +1234,9 @@ class BTreeGraphIndex(object):
             missing_keys.update(keys)
             return set()
         if ref_list_num >= self.node_ref_lists:
-            raise ValueError('No ref list %d, index has %d ref lists'
-                             % (ref_list_num, self.node_ref_lists))
+            raise ValueError(
+                f"No ref list {ref_list_num}, index has {self.node_ref_lists} ref lists"
+            )
 
         # The main trick we are trying to accomplish is that when we find a
         # key listing its parents, we expect that the parent key is also likely
@@ -1282,7 +1292,7 @@ class BTreeGraphIndex(object):
                 next_parents_to_check = set()
                 for key in parents_to_check:
                     if key in node:
-                        value, refs = node[key]
+                        _value, refs = node[key]
                         parent_keys = refs[ref_list_num]
                         parent_map[key] = parent_keys
                         next_parents_to_check.update(parent_keys)
@@ -1317,8 +1327,9 @@ class BTreeGraphIndex(object):
         # parents_not_on_page could have been found on a different page, or be
         # known to be missing. So cull out everything that has already been
         # found.
-        search_keys = parents_not_on_page.difference(
-            parent_map).difference(missing_keys)
+        search_keys = parents_not_on_page.difference(parent_map).difference(
+            missing_keys
+        )
         return search_keys
 
     def iter_entries_prefix(self, keys):
@@ -1383,7 +1394,7 @@ class BTreeGraphIndex(object):
                     key_dict[key[-1]] = key_value
         if self._key_length == 1:
             for key in keys:
-                index._sanity_check_key(self, key)
+                _mod_index._sanity_check_key(self, key)
                 try:
                     if self.node_ref_lists:
                         value, node_refs = nodes[key]
@@ -1393,8 +1404,7 @@ class BTreeGraphIndex(object):
                 except KeyError:
                     pass
             return
-        for entry in index._iter_entries_prefix(self, nodes_by_key, keys):
-            yield entry
+        yield from _mod_index._iter_entries_prefix(self, nodes_by_key, keys)
 
     def key_count(self):
         """Return an estimate of the number of keys in this index.
@@ -1423,45 +1433,46 @@ class BTreeGraphIndex(object):
         :return: An offset, data tuple such as readv yields, for the unparsed
             data. (which may be of length 0).
         """
-        signature = bytes[0:len(self._signature())]
+        signature = bytes[0 : len(self._signature())]
         if not signature == self._signature():
-            raise index.BadIndexFormatSignature(self._name, BTreeGraphIndex)
-        lines = bytes[len(self._signature()):].splitlines()
+            raise _mod_index.BadIndexFormatSignature(self._name, BTreeGraphIndex)
+        lines = bytes[len(self._signature()) :].splitlines()
         options_line = lines[0]
         if not options_line.startswith(_OPTION_NODE_REFS):
-            raise index.BadIndexOptions(self)
+            raise _mod_index.BadIndexOptions(self)
         try:
-            self.node_ref_lists = int(options_line[len(_OPTION_NODE_REFS):])
-        except ValueError:
-            raise index.BadIndexOptions(self)
+            self.node_ref_lists = int(options_line[len(_OPTION_NODE_REFS) :])
+        except ValueError as e:
+            raise _mod_index.BadIndexOptions(self) from e
         options_line = lines[1]
         if not options_line.startswith(_OPTION_KEY_ELEMENTS):
-            raise index.BadIndexOptions(self)
+            raise _mod_index.BadIndexOptions(self)
         try:
-            self._key_length = int(options_line[len(_OPTION_KEY_ELEMENTS):])
-        except ValueError:
-            raise index.BadIndexOptions(self)
+            self._key_length = int(options_line[len(_OPTION_KEY_ELEMENTS) :])
+        except ValueError as e:
+            raise _mod_index.BadIndexOptions(self) from e
         options_line = lines[2]
         if not options_line.startswith(_OPTION_LEN):
-            raise index.BadIndexOptions(self)
+            raise _mod_index.BadIndexOptions(self)
         try:
-            self._key_count = int(options_line[len(_OPTION_LEN):])
-        except ValueError:
-            raise index.BadIndexOptions(self)
+            self._key_count = int(options_line[len(_OPTION_LEN) :])
+        except ValueError as e:
+            raise _mod_index.BadIndexOptions(self) from e
         options_line = lines[3]
         if not options_line.startswith(_OPTION_ROW_LENGTHS):
-            raise index.BadIndexOptions(self)
+            raise _mod_index.BadIndexOptions(self)
         try:
-            self._row_lengths = [int(length) for length in
-                                 options_line[len(_OPTION_ROW_LENGTHS):].split(
-                                     b',')
-                                 if length]
-        except ValueError:
-            raise index.BadIndexOptions(self)
+            self._row_lengths = [
+                int(length)
+                for length in options_line[len(_OPTION_ROW_LENGTHS) :].split(b",")
+                if length
+            ]
+        except ValueError as e:
+            raise _mod_index.BadIndexOptions(self) from e
         self._compute_row_offsets()
 
         # calculate the bytes we have processed
-        header_end = (len(signature) + sum(map(len, lines[0:4])) + 4)
+        header_end = len(signature) + sum(map(len, lines[0:4])) + 4
         return header_end, bytes[header_end:]
 
     def _read_nodes(self, nodes):
@@ -1482,7 +1493,7 @@ class BTreeGraphIndex(object):
         ranges = []
         base_offset = self._base_offset
         for index in nodes:
-            offset = (index * _PAGE_SIZE)
+            offset = index * _PAGE_SIZE
             size = _PAGE_SIZE
             if index == 0:
                 # Root node - special case
@@ -1495,23 +1506,27 @@ class BTreeGraphIndex(object):
                     num_bytes = len(bytes)
                     self._size = num_bytes - base_offset
                     # the whole thing should be parsed out of 'bytes'
-                    ranges = [(start, min(_PAGE_SIZE, num_bytes - start))
-                              for start in range(
-                                  base_offset, num_bytes, _PAGE_SIZE)]
+                    ranges = [
+                        (start, min(_PAGE_SIZE, num_bytes - start))
+                        for start in range(base_offset, num_bytes, _PAGE_SIZE)
+                    ]
                     break
             else:
                 if offset > self._size:
-                    raise AssertionError('tried to read past the end'
-                                         ' of the file %s > %s'
-                                         % (offset, self._size))
+                    raise AssertionError(
+                        "tried to read past the end of the file {} > {}".format(
+                            offset, self._size
+                        )
+                    )
                 size = min(size, self._size - offset)
             ranges.append((base_offset + offset, size))
         if not ranges:
             return
         elif bytes is not None:
             # already have the whole file
-            data_ranges = [(start, bytes[start:start + size])
-                           for start, size in ranges]
+            data_ranges = [
+                (start, bytes[start : start + size]) for start, size in ranges
+            ]
         elif self._file is None:
             data_ranges = self._transport.readv(self._name, ranges)
         else:
@@ -1528,12 +1543,11 @@ class BTreeGraphIndex(object):
                     continue
             bytes = zlib.decompress(data)
             if bytes.startswith(_LEAF_FLAG):
-                node = self._leaf_factory(bytes, self._key_length,
-                                          self.node_ref_lists)
+                node = self._leaf_factory(bytes, self._key_length, self.node_ref_lists)
             elif bytes.startswith(_INTERNAL_FLAG):
                 node = _InternalNode(bytes)
             else:
-                raise AssertionError("Unknown node type for %r" % bytes)
+                raise AssertionError("Unknown node type for {!r}".format(bytes))
             yield offset // _PAGE_SIZE, node
 
     def _signature(self):
@@ -1550,15 +1564,16 @@ class BTreeGraphIndex(object):
             # We shouldn't be reading anything anyway
             start_node = 1
         node_end = self._row_offsets[-1]
-        for node in self._read_nodes(list(range(start_node, node_end))):
+        for _node in self._read_nodes(list(range(start_node, node_end))):
             pass
 
 
 _gcchk_factory = _LeafNode
 
 try:
-    from . import _btree_serializer_pyx as _btree_serializer
-    _gcchk_factory = _btree_serializer._parse_into_chk
+    from . import _btree_serializer_pyx as _btree_serializer  # type: ignore
+
+    _gcchk_factory = _btree_serializer._parse_into_chk  # type: ignore
 except ImportError as e:
     osutils.failed_to_load_extension(e)
     from . import _btree_serializer_py as _btree_serializer
